"""
Core functionality for the Dolze Templates library.

This module provides the main classes and functions for working with templates.
"""

from .template_variables import TemplateVariables

__all__ = [
    "TemplateVariables",
]
